#include <gst/gst.h>

int
main (int argc, char *argv[])
{
  GstElement *bin;
  gchar *pipe_str;
  GError *error = NULL;

  gst_init (&argc, &argv);

  if (argc != 2) {
    g_print ("usage: %s <mp3 file>\n", argv[0]);
    return 1;
  }

  g_print ("Creating pipeline\n");
  
  /* Create the pipeline description */
  pipe_str = g_strdup_printf (
              "filesrc location=%s ! mad ! audioconvert ! alsasink", 
              argv[1]);

  /* use gst_parse_launch to create the pipeline */
  bin = gst_parse_launch (pipe_str, &error);
  /* Check if any errors occurred */
  if (error != NULL) {
    g_print ("Error constructing pipeline: %s\n", error->message);
    g_error_free (error);
    return 1;
  }
  g_return_val_if_fail (bin != NULL, 1); 

  g_print ("Starting playback\n");
  
  /* start playing */
  gst_element_set_state (bin, GST_STATE_PLAYING);

  /* Sleep for 5 seconds while things play */
  g_usleep (5 * G_USEC_PER_SEC);

  g_print ("Ending playback and stopping pipeline\n");

  /* stop the pipeline */
  gst_element_set_state (bin, GST_STATE_NULL);

  /* Unreffing the bin will clean up all its children too */
  gst_object_unref (bin);
  
  return 0;
}
