#include <gst/gst.h>

int
main (int argc, char *argv[])
{
  GstElement *bin, *filesrc, *decoder, *audioconvert, *audiosink;

  gst_init (&argc, &argv);

  if (argc != 2) {
    g_print ("usage: %s <mp3 file>\n", argv[0]);
    return 1;
  }

  g_print ("Creating pipeline\n");
  
  /* create a new bin to hold the elements */
  bin = gst_pipeline_new ("pipeline");
  g_assert (bin);

  /* create a disk reader */
  filesrc = gst_element_factory_make ("filesrc", "disk_source");
  g_assert (filesrc);
  g_object_set (G_OBJECT (filesrc), "location", argv[1], NULL);

  /* now it's time to get the decoder */
  decoder = gst_element_factory_make ("mad", "decode");
  if (!decoder) {
    g_print ("could not find element \"mad\"\n");
    return -1;
  }

  audioconvert = gst_element_factory_make ("audioconvert", "audioconvert");
  g_assert (audioconvert);

  /* and an audio sink */
  audiosink = gst_element_factory_make ("alsasink", "play_audio");
  g_assert (audiosink);

  /* add objects to the main pipeline */
  gst_bin_add_many (GST_BIN (bin), filesrc, decoder, audioconvert, audiosink, NULL);

  /* link the elements */
  gst_element_link_many (filesrc, decoder, audioconvert, audiosink, NULL);

  g_print ("Starting playback\n");
  
  gst_element_set_state (bin, GST_STATE_PLAYING);

  g_usleep (5 * G_USEC_PER_SEC);

  g_print ("Ending playback and stopping pipeline\n");
  
  gst_element_set_state (bin, GST_STATE_NULL);

  /* Unreffing the bin will clean up all its children too */
  gst_object_unref (bin);
  
  return 0;
}
